#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
CedictParser and CedictEntry unit tests
"""

import unittest
from cedict_utils.cedict import CedictParser, CedictEntry


class TestCedictParser(unittest.TestCase):
    """Main parser test case"""

    def test_remove_lines_with_comments(self):
        an_entry = (
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/\n"
        )
        lines = [
            "# CC-CEDICT\n",
            "# Community Chinese-English dictionary.\n",
            "# \n",
            "# Creative Commons Attribution-Share Alike 3.0\n",
            "#! publisher=MDBG\n",
            an_entry,
        ]
        parser = CedictParser(lines=lines)

        parser._filter_comments()

        self.assertEqual(len(parser.lines), 1)
        self.assertEqual(an_entry, parser.lines[0])

    def test_remove_new_lines(self):
        original_lines = [
            "# CC-CEDICT\n",
            "# Community Chinese-English dictionary.\n",
            "# \n",
            "#! publisher=MDBG\n",
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/\n",
        ]
        expected_lines = [
            "# CC-CEDICT",
            "# Community Chinese-English dictionary.",
            "# ",
            "#! publisher=MDBG",
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/",
        ]
        parser = CedictParser(lines=original_lines)

        parser._filter_new_lines()

        self.assertCountEqual(parser.lines, expected_lines)

    def test_remove_empty_lines(self):
        original_lines = [
            " ",
            "",
            "# \n",
            "#! publisher=MDBG\n",
            "\n",
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/\n",
        ]
        expected_lines = [
            "# \n",
            "#! publisher=MDBG\n",
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/\n",
        ]
        parser = CedictParser(lines=original_lines)

        parser._filter_empty_entries()

        self.assertCountEqual(parser.lines, expected_lines)

    def test_sanitize_lines(self):
        """ Test if all filters are being applied """
        original_lines = [
            " ",
            "",
            "# \n",
            "#! publisher=MDBG\n",
            "\n",
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/\n",
        ]
        expected_lines = [
            "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/"
        ]
        parser = CedictParser(lines=original_lines)

        parser._sanitize()

        self.assertCountEqual(parser.lines, expected_lines)

    def test_parse_entry(self):
        lines = ["K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/\n"]
        parser = CedictParser(lines=lines)

        entries = parser.parse()

        self.assertTrue(type(entries[0]), CedictEntry)


class TestCedictEntry(unittest.TestCase):
    """CedictEntry test case"""

    def test_make_entry_from_cedict_line(self):
        line = "K書 K书 [K shu1] /to cram (Taiwan,khè su 齧書, lit./ 啃書|啃书[ken3 shu1]/"
        traditional = "K書"
        simplified = "K书"
        pinyin = "K shu1"
        meanings = ["to cram (Taiwan,khè su 齧書, lit.", " 啃書|啃书[ken3 shu1]"]

        entry = CedictEntry.make(line)

        self.assertTrue(type(entry), CedictEntry)
        self.assertEqual(traditional, entry.traditional)
        self.assertEqual(simplified, entry.simplified)
        self.assertEqual(pinyin, entry.pinyin)
        self.assertEqual(meanings, entry.meanings)

    def test_make_entry_from_cedict_line_one_meaning(self):
        line = "K書 K书 [K shu1] /啃書|啃书[ken3 shu1]/"
        meanings = ["啃書|啃书[ken3 shu1]"]

        entry = CedictEntry.make(line)

        self.assertEqual(meanings, entry.meanings)

    def test_make_entry_from_cedict_line_replaces_double_in_quotes_in_meaning(self):
        line = 'K書 K书 [K shu1] /啃"h "u1/'
        meanings = ["啃'h 'u1"]

        entry = CedictEntry.make(line)

        self.assertEqual(meanings, entry.meanings)

    # def test_expand_abbreviations(self):
    #     meaning = "this abbr. and fig. humble expr. Tw."
    #     expected = "this abbreviation and metaphorically " \
    #                "humble expression used in Taiwan"

    #     result = expand_abbreviations(meaning)

    #     self.assertEqual(expected, result)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Parser class for cedict Chinese-English dictionary files
"""

import os
import os.path
import io
import logging
import pickle

from os import path

BASE_PATH = path.abspath(path.dirname(__file__))
DATA_DIR = os.path.join(BASE_PATH, "data")
CEDICT_PATH = os.path.join(DATA_DIR, "cedict_ts.u8")
DATA_PATH = os.path.join(DATA_DIR, "dump.dat")

class CedictParser:
    """Parser class. Reads a cedict file and return a list of
    CedictEntry instances with each line processed.
    """

    filters = ["_filter_comments", "_filter_new_lines", "_filter_empty_entries"]

    def __init__(self, lines=None, file_path=None, lines_count=None):
        self.lines = lines or []
        self.lines_count = lines_count
        
        if not file_path and os.path.isfile(DATA_PATH):
            self.lines = pickle.load( open( DATA_PATH, "rb" ) )
        elif file_path:
            self.read_file(file_path)
        else:
            self.read_file(file_path=CEDICT_PATH)

    def read_file(self, file_path):
        """Import the cedict file sanitizing each entry"""
        with io.open(
            file_path, "r", encoding="utf-8"
        ) as file_handler:
            if self.lines_count:
                logging.info("Loaded %s lines of the dictionary", self.lines_count)
            self.lines = file_handler.readlines()
            self._sanitize()
            pickle.dump( self.lines, open( DATA_PATH, "wb" ) )

    def _sanitize(self):
        from operator import methodcaller

        # f = methodcaller('_filter_comments')
        # f(b) returns b._filter_comments().
        for fun in self.filters:
            caller = methodcaller(fun)
            caller(self)

    def _filter_comments(self):
        """ remove lines starting with # or #! """
        self.lines = [line for line in self.lines if not line.startswith(("#"))]

    def _filter_new_lines(self):
        self.lines = [line.rstrip("\n") for line in self.lines]

    def _filter_empty_entries(self):
        self.lines = [line for line in self.lines if line.strip()]

    def parse(self):
        """ Parse Cedict lines and return a list of CedictEntry items """
        result = []
        for line in self.lines[: self.lines_count]:
            entry = CedictEntry.make(line)
            result.append(entry)
        return result


class CedictEntry:  # pylint: disable=too-few-public-methods
    """A representation of a cedict entry

    Keyword arguments:
    traditional -- entry in traditional hanzi
    simplified -- entry in simplified hanzi
    pinyin -- entry pronunciation with tone numbers
    meanings -- list of different meanings for an entry
    raw_line -- the original full line
    """

    def __init__(self, **kwargs):
        self.traditional = kwargs.get("traditional", "")
        self.simplified = kwargs.get("simplified", "")
        self.pinyin = kwargs.get("pinyin", "")
        self.meanings = kwargs.get("meanings", "")
        self.raw_line = kwargs.get("raw_line", "")

    @classmethod
    def make(cls, line):
        """ Generates an entry from a Cedict file line data """
        hanzis = line.partition("[")[0].split(" ", 1)
        keywords = dict(
            meanings=line.partition("/")[2]
            .replace('"', "'")
            .rstrip("/")
            .strip()
            .split("/"),
            traditional=hanzis[0].strip(" "),
            simplified=hanzis[1].strip(" "),
            # Take the content in between the two brackets
            pinyin=line.partition("[")[2].partition("]")[0],
            raw_line=line,
        )
        return cls(**keywords)

    def __str__(self):
        return "{} ({}) - {}".format(self.simplified, self.traditional, self.pinyin)
